import ply.yacc as yacc
import stringer_object as str_obj

class StringerParser:
    start = 'program'
    
    def __init__(self, strLex):
        self.tokens = strLex.tokens
        self.lexer = strLex.lexer
    
    def p_empty(self, p):
        'empty :'
        pass
        
    def p_program_empty(self, p):
        'program : empty'
        p[0] = str_obj.Program({})
        
    def p_program(self, p):
        '''program : MacroGrp macro_defs program
                 | TextGrp text_defs program
                 | Comment program'''
        if len(p) >= 4:
            p[3].add(p[2])
            p[0] = p[3]
        else:
            p[0] = p[2]
        
    def p_macro_defs(self, p):
        '''macro_defs : macro_def macro_defs
                      | macro_def'''
        if len(p) >= 3:
            p[0] = p[2]
        else:
            p[0] = str_obj.Program({})
            
        for key in p[1].keys():
            p[0].macros[key] = p[1][key]
        
    def p_macro_def(self, p):
        'macro_def : Variable def_args Equal macro_body'
        p[0] = {}
        p[4].name = p[1]
        p[4].arg_names = p[2]
        p[0][p[1]] = p[4]
        
    def p_macro_body_number(self, p):
        'macro_body : Number'
        p[0] = str_obj.MacroNumber(p[1])
        
    def p_macro_body_pair(self, p):
        'macro_body : LParen Number Comma Number RParen'
        p[0] = str_obj.MacroPair(p[2], p[4])
        
    def p_macro_body_array(self, p):
        'macro_body : LSquare array_items RSquare'
        p[0] = str_obj.MacroArray(p[2])
        
    def p_macro_body_text(self, p):
        'macro_body : text'
        p[0] = p[1]
        
    def p_macro_body_code(self, p):
        'macro_body : CodeQuote Code Quote'
        p[0] = str_obj.Macro(p[2])
        
    def p_macro_def_dict(self, p):
        'macro_def : Variable def_args Equal LBrace dict_body RBrace'
        p[0] = {}
        for key in p[5].keys():
            gKey = '%s_%s' % (p[1], key)
            p[0][gKey] = p[5][key]
            
    def p_dict_body(self, p):
        '''dict_body : macro_def Comma dict_body
                     | macro_def'''
        if len(p) >= 4:
            p[0] = p[3]
        else:
            p[0] = {}
        for key in p[1].keys():
            p[0][key] = p[1][key]
        
    def p_array_items(self, p):
        '''array_items : array_item Comma array_items
                       | array_item'''
        if len(p) >= 4:
            p[0] = p[3]
        else:
            p[0] = []
            
        p[0].insert(0, p[1])
        
    def p_array_item_number(self, p):
        'array_item : Number'
        p[0] = str_obj.MacroNumber(p[1])
        
    def p_array_item_text(self, p):
        'array_item : text'
        p[0] = p[1]
    
    def p_def_args(self, p):
        '''def_args : Variable def_args
                    | empty'''
        if len(p) >= 3:
            p[0] = p[2]
            p[0].insert(0, p[1])
        else:
            p[0] = []    
     
    def p_text_defs(self, p):
        '''text_defs : text_def text_defs
                     | text_def'''
        if len(p) >= 3:
            p[0] = p[2]
        else:
            p[0] = str_obj.Program({})
            
        if p[1] != None:
            for key in p[1].keys():
                p[0].texts[key] = p[1][key]
    
    def p_text_def(self, p):
        'text_def : Variable Equal text'
        p[0] = {}
        p[3].name = p[1]
        p[0][p[1]] = p[3]
        
    def p_text(self, p):
        '''text : Quote text_body Quote
                | macro_call'''
        if len(p) >= 4:
            p[0] = p[2]
        else:
            p[0] = str_obj.MacroText()
            p[0].add(p[1])
        
    def p_text_def_empty(self, p):
        'text_def : Variable Equal Quote Quote'
        m = str_obj.MacroText()
        m.add("")
        p[0] = {}
        p[0][p[1]] = m
        
    def p_text_body(self, p):
        '''text_body : text_item text_body
                     | text_item'''
        if len(p) >= 3:
            p[0] = p[2]
        else:
            p[0] = str_obj.MacroText()
        p[0].add(p[1])
        
    def p_text_item(self, p):
        '''text_item : String
                     | macro_call'''
        p[0] = p[1]
        
    def p_macro_call(self, p):
        'macro_call : LAngle macro_name macro_args RAngle'
        p[0] = str_obj.InvokeMacro(p[2], p[3])
    
    def p_macro_name(self, p):
        '''macro_name : Variable
                      | MacroDot
                      | MacroPercent
                      | MacroLevel'''
        p[0] = p[1]
        
    def p_macro_args(self, p):
        '''macro_args : macro_arg macro_args
                      | empty'''
        if len(p) >= 3:
            p[0] = p[2]
            p[0].insert(0, p[1])
        else:
            p[0] = []
        
    def p_macro_arg(self, p):
        '''macro_arg : Number
                     | Variable
                     | text'''
        p[0] = p[1]
        
        
    def build(self, **kwargs):
        self.parser = yacc.yacc(module=self, **kwargs)
    
    def run(self, data):
        return self.parser.parse(data, lexer=self.lexer)